<?xml version="1.0"?>
<!--
 Derived from Conformance1-5.xsl.

 Copyright (c) 2001, 2006 Rick Jelliffe and Academia Sinica Computing Center, Taiwan

 This software is provided 'as-is', without any express or implied warranty. 
 In no event will the authors be held liable for any damages arising from 
 the use of this software.

 Permission is granted to anyone to use this software for any purpose, 
 including commercial applications, and to alter it and redistribute it freely,
 subject to the following restrictions:

 1. The origin of this software must not be misrepresented; you must not claim
 that you wrote the original software. If you use this software in a product, 
 an acknowledgment in the product documentation would be appreciated but is 
 not required.

 2. Altered source versions must be plainly marked as such, and must not be 
 misrepresented as being the original software.

 3. This notice may not be removed or altered from any source distribution.
-->
<!-- Ideas nabbed from schematrons by Francis N., Miloslav N. and David C. -->
<!-- The command-line parameters are:
  			phase           NMTOKEN | "#ALL" (default) Select the phase for validation
    		allow-foreign   "true" | "false" (default)   Pass non-Schematron elements  and rich markup  to the generated stylesheet
            diagnose= true | false|yes|no    Add the diagnostics to the assertion test in reports (yes|no are obsolete)
            generate-paths=true|false|yes|no   generate the @location attribute with XPaths (yes|no are obsolete)
            sch.exslt.imports semi-colon delimited string of filenames for some EXSLT implementations          
   		 optimize        "visit-no-attributes"     Use only when the schema has no attributes as the context nodes
		 generate-fired-rule "true"(default) | "false"  Generate fired-rule elements
            
-->
<!-- The following command line options are available in the resulting XSLT for SBR
			full-event	Produce a complete MessageEvent structure, including the correct maximum severity
			document-sequence-number	Sequence number to be included in location indications
-->
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:axsl="http://www.w3.org/1999/XSL/TransformAlias"
                xmlns:schold="http://www.ascc.net/xml/schematron"
                xmlns:iso="http://purl.oclc.org/dsdl/schematron"
                xmlns:sbdm="http://sbr.gov.au/comn/sbdm.02.data"
                xmlns:isns="http://www.ato.gov.au/IS/2009/02"
                xmlns:cl="http://sbr.gov.au/comn/codelist.02.data"
                exclude-result-prefixes="schold cl">
  <!-- Select the import statement and adjust the path as 
   necessary for your system.
   If not XSLT2 then also remove svrl:active-pattern/@document="{document-uri()}" from process-pattern()
-->
  <!--
<xsl:import href="iso_schematron_skeleton_for_saxon.xsl"/>
-->
  <xsl:import href="iso_schematron_skeleton_for_xslt1.xsl"/>
  <!--
<xsl:import href="iso_schematron_skeleton.xsl"/>
<xsl:import href="skeleton1-5.xsl"/>
<xsl:import href="skeleton1-6.xsl"/>
-->
  <xsl:param name="diagnose">true</xsl:param>
  <xsl:param name="phase">
    <xsl:choose>
      <!-- Handle Schematron 1.5 and 1.6 phases -->
      <xsl:when test="//schold:schema/@defaultPhase">
        <xsl:value-of select="//schold:schema/@defaultPhase"/>
      </xsl:when>
      <!-- Handle ISO Schematron phases -->
      <xsl:when test="//iso:schema/@defaultPhase">
        <xsl:value-of select="//iso:schema/@defaultPhase"/>
      </xsl:when>
      <xsl:otherwise>#ALL</xsl:otherwise>
    </xsl:choose>
  </xsl:param>
  <xsl:param name="allow-foreign">false</xsl:param>
  <xsl:param name="generate-paths">true</xsl:param>
  <xsl:param name="generate-fired-rule">true</xsl:param>
  <xsl:param name="optimize"/>
  <xsl:param name="output-encoding"/>
  <!-- e.g. saxon file.xml file.xsl "sch.exslt.imports=.../string.xsl;.../math.xsl" -->
  <xsl:param name="sch.exslt.imports"/>
  <!-- ================================================================ -->
  <!-- Determine the output format required. The idea is that the resulting structures should be directly deserialisable -->
  <!-- into the classes generated by processing the SBR WSDLs -->
  <xsl:template name="process-root">
    <xsl:param name="contents"/>
    <axsl:variable name="results">
      <xsl:apply-templates mode="do-schema-p"/>
      <xsl:copy-of select="$contents"/>
    </axsl:variable>
    <axsl:if test="$results">
      <axsl:choose>
        <axsl:when test="$full-event='true'">
          <sbdm:MessageEvent>
            <sbdm:Message.Event.MaximumSeverity.Code>
              <axsl:choose>
                <axsl:when test="$results//sbdm:Message.Event.Item.Severity.Code[.='Error']">Error</axsl:when>
                <axsl:when test="$results//sbdm:Message.Event.Item.Severity.Code[.='Warning']">Warning</axsl:when>
                <axsl:otherwise>Information</axsl:otherwise>
              </axsl:choose>
            </sbdm:Message.Event.MaximumSeverity.Code>
            <sbdm:MessageEventItems>
              <axsl:copy-of select="$results"/>
            </sbdm:MessageEventItems>
          </sbdm:MessageEvent>
        </axsl:when>
        <axsl:otherwise>
          <sbdm:MessageEventItems>
            <axsl:copy-of select="$results"/>
          </sbdm:MessageEventItems>
        </axsl:otherwise>
      </axsl:choose>
    </axsl:if>
  </xsl:template>
  <!-- Define the variables to be available at run-time. -->
  <xsl:template name="process-prolog">
    <!-- Variable to support the use of an apostrophe in country names. -->
    <xsl:text disable-output-escaping="yes">&lt;xsl:variable name="Apostrophe"&gt;&#39;&lt;/xsl:variable&gt;</xsl:text>

    <!-- Function to support "castable" style XSLT 2.0 functionality. -->
    <!-- Determines if the given variable conforms to the required type. -->
    <xsl:text disable-output-escaping="yes">
		&lt;msxsl:script language="C#" implements-prefix="function"&gt;

		public bool IsDate(string dateToCheck)
		{
			DateTime dt;
			return DateTime.TryParse(dateToCheck, out dt);
		}

		public bool IsInt(string intToCheck)
		{
			Int32 i;
			return Int32.TryParse(intToCheck, out i);
		}

		public bool IsLong(string longToCheck)
		{
			long l;
			return long.TryParse(longToCheck, out l);
		}

    public bool IsDate1GreaterThanDate2(string date1, string date2)
    {
        bool b = false;
        
        if (IsDate(date1))
        {
          if (IsDate(date2))
          {
            if (DateTime.Parse(date1) > DateTime.Parse(date2))
            {
                b = true;
            }
          }
        }

        return b;
    }

    public bool IsDate1GreaterOrEqualToDate2(string date1, string date2)
    {
        bool b = false;

        if (IsDate(date1))
        {
          if (IsDate(date2))
          {
            if (DateTime.Parse(date1) >= DateTime.Parse(date2))
            {
                b = true;
            }
          }
        }

        return b;
    }

    public int GetDateYear(string date)
    {
      int year = 0;

      if (IsDate(date))
      {
        year = DateTime.Parse(date).Year;
      }

      return year;
    }

    public int DateToFinancialYear(DateTime date)
    {
        return DateToFinancialYear(date, 7);
    }

    public int DateToFinancialYear(DateTime date, int financialYearStartMonth)
    {
        return financialYearStartMonth > date.Month ? date.Year : date.Year + 1;
    }

    public bool ValidateABN(string abn)
    {
      string cleanABN = abn.Replace(" ", string.Empty);
           
      if (cleanABN.Length != 11)
          return false;

      Int64 result;
      bool convertOK = Int64.TryParse(cleanABN, out result);
      if (!convertOK)
      {
          return false;
      }

      int[] weighingFactors = { 10, 1, 3, 5, 7, 9, 11, 13, 15, 17, 19 };
      int weightedSum = (Int32.Parse(cleanABN[0].ToString()) - 1) * weighingFactors[0];

      for (int i = cleanABN.Length; i > 1; i--)
      {
          weightedSum += Int32.Parse(cleanABN[i - 1].ToString()) * weighingFactors[i - 1];
      }

      int remainder = weightedSum % 89;
      return (remainder == 0);
    }

    public bool ValidateTAN(string tan)
    {
      string cleanTAN = tan.Replace(" ", string.Empty);
      if (cleanTAN.Length != 8)
        return false;

      Int64 result;
      bool convertOK = Int64.TryParse(cleanTAN, out result);
      if (!convertOK)
      {
        return false;
      }

      int[] weighingFactors = {7,9,8,4,6,3,5,1};
      int weightedSum = 0;
      for (int i = cleanTAN.Length; i > 0; i--)
      {
        weightedSum += Int32.Parse(cleanTAN[i-1].ToString()) * weighingFactors[i-1];
      }

      int remainder = weightedSum % 11;
      return (remainder == 0);
    }

    public static bool ValidateTFN(string tfn)
    {
      bool isValid = false;
      string cleanTFN = tfn.Replace(" ", string.Empty).TrimStart(new char[] {'0'});
      
      if (cleanTFN.Length == 8 || cleanTFN.Length == 9)
      {
          cleanTFN = cleanTFN.Length == 8 ? cleanTFN.Insert(7, "0") : cleanTFN;

          Int64 result;
          bool convertOK = Int64.TryParse(cleanTFN, out result);

          if (convertOK)
          {
              int[] weighingFactors = { 1, 4, 3, 7, 5, 8, 6, 9, 10 };
              int weightedSum = 0;

              for (int i = cleanTFN.Length; i > 0; i--)
              {
                  weightedSum += Int32.Parse(cleanTFN[i - 1].ToString()) * weighingFactors[i - 1];
              }

              isValid = (weightedSum % 11 == 0);
          }
      }

      return isValid;
    }

    public static bool ValidateACN(string acn)
    {
      string cleanACN = acn.Replace(" ", string.Empty);
      if (cleanACN.Length != 9)
      {
          return false;
      }

      Int64 result;
      bool convertOK = Int64.TryParse(cleanACN, out result);
      if (!convertOK)
      {
          return false;
      }

      int[] weighingFactors = { 8, 7, 6, 5, 4, 3, 2, 1 };
      int checkDigit = Int32.Parse(cleanACN[8].ToString());

      int weightedSum = 0;
      for (int i = cleanACN.Length - 1; i > 0; i--)
      {
          weightedSum += Int32.Parse(cleanACN[i - 1].ToString()) * weighingFactors[i - 1];
      }

      int remainder = weightedSum % 10;
      return checkDigit == (10 - remainder) % 10;
    }

        public int GetFBTYear(string date)
        {
            int fbtYear = 0;
            if (IsDate(date))
            {
                DateTime dateTime = DateTime.Parse(date);
                int year = dateTime.Year;
                if (dateTime.Month > 3)
                {
                    fbtYear = year + 1;
                }
                else
                {
                    fbtYear = year;
                }
            }
            return fbtYear;
        }
        public int GetCurrentFBTYear()
        {
            return GetFBTYear(DateTime.Now.ToString("yyyy-MM-dd"));
        }

        
		&lt;/msxsl:script&gt;
		</xsl:text>

    <axsl:output method="xml" omit-xml-declaration="no" standalone="yes" indent="yes">
      <xsl:if test=" string-length($output-encoding) &gt; 0">
        <xsl:attribute name="encoding">
          <xsl:value-of select=" $output-encoding"/>
        </xsl:attribute>
      </xsl:if>
    </axsl:output>
    <axsl:param name="full-event">false</axsl:param>
    <axsl:param name="document-sequence-number">1</axsl:param>
  </xsl:template>
  <!-- Rely on the skeleton processing for assert -->
  <xsl:template name="process-assert">
    <xsl:param name="diagnostics"/>
    <xsl:param name="flag"/>
    <xsl:if test="$diagnose = 'yes' or $diagnose= 'true' ">
      <!-- true/false is the new way -->
      <xsl:call-template name="diagnosticsSplit">
        <xsl:with-param name="str" select="$diagnostics"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <!-- Rely on the skeleton processing for report -->
  <xsl:template name="process-report">
    <xsl:param name="diagnostics"/>
    <xsl:param name="flag"/>
    <xsl:if test="$diagnose = 'yes' or $diagnose='true' ">
      <!-- true/false is the new way -->
      <xsl:call-template name="diagnosticsSplit">
        <xsl:with-param name="str" select="$diagnostics"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <!-- Most of the work is done by the skeleton in copying across the contents of the diagnostic element from the schematron file -->
  <xsl:template name="process-diagnostic">
    <xsl:param name="id"/>
    <!-- Rich parameters -->
    <xsl:param name="fpi" />
    <xsl:param name="icon" />
    <xsl:param name="lang" />
    <xsl:param name="see" />
    <xsl:param name="space" />

    <sbdm:MessageEventItem>
      <sbdm:Message.Event.Item.Error.Code>
        <xsl:value-of select="sbdm:MessageEventItem/sbdm:Message.Event.Item.Error.Code/text()" />
      </sbdm:Message.Event.Item.Error.Code>
      
      <xsl:choose>
        <xsl:when test="string-length( sbdm:MessageEventItem/sbdm:Message.Event.Item.Severity.Code/text() ) &gt; 0">
          <sbdm:Message.Event.Item.Severity.Code>
            <xsl:value-of select="sbdm:MessageEventItem/sbdm:Message.Event.Item.Severity.Code/text()" />
          </sbdm:Message.Event.Item.Severity.Code>
        </xsl:when>
        <xsl:otherwise>
          <sbdm:Message.Event.Item.Severity.Code>Error</sbdm:Message.Event.Item.Severity.Code>
        </xsl:otherwise>
      </xsl:choose>

      <xsl:if test="string-length( sbdm:MessageEventItem/sbdm:Message.Event.Item.Short.Description/text() ) &gt; 0">
        <sbdm:Message.Event.Item.Short.Description>
          <xsl:value-of select="sbdm:MessageEventItem/sbdm:Message.Event.Item.Short.Description/text()" />
        </sbdm:Message.Event.Item.Short.Description>
      </xsl:if>

      <xsl:if test="string-length( sbdm:MessageEventItem/sbdm:Message.Event.Item.Detailed.Description/text() ) &gt; 0">
        <sbdm:Message.Event.Item.Detailed.Description>
          <xsl:value-of select="sbdm:MessageEventItem/sbdm:Message.Event.Item.Detailed.Description/text()" />
        </sbdm:Message.Event.Item.Detailed.Description>
      </xsl:if>

      <!--<xsl:if test="count(sbdm:MessageEventItem/sbdm:Parameters) = 1">-->
      <sbdm:Parameters>
        <xsl:for-each select="sbdm:MessageEventItem/sbdm:Parameters/sbdm:Parameter">
          <sbdm:Parameter>
            <sbdm:Message.Event.Item.Parameter.Identifier>
              <xsl:value-of select="sbdm:Message.Event.Item.Parameter.Identifier/text()" />
            </sbdm:Message.Event.Item.Parameter.Identifier>
            <sbdm:Message.Event.Item.Parameter.Text>
              <xsl:copy-of select ="sbdm:Message.Event.Item.Parameter.Text/node()"/>
            </sbdm:Message.Event.Item.Parameter.Text>
          </sbdm:Parameter>
        </xsl:for-each>
        <sbdm:Parameter>
          <sbdm:Message.Event.Item.Parameter.Identifier>RuleIdentifier</sbdm:Message.Event.Item.Parameter.Identifier>
          <sbdm:Message.Event.Item.Parameter.Text>
            <xsl:value-of select="$id" />
          </sbdm:Message.Event.Item.Parameter.Text>
        </sbdm:Parameter>
      </sbdm:Parameters>
      <!--</xsl:if>-->

      <sbdm:Locations>
        <sbdm:Location>
          <sbdm:BusinessDocument.Sequence.Number>
            <axsl:value-of select="$document-sequence-number"/>
          </sbdm:BusinessDocument.Sequence.Number>
          <sbdm:Message.Event.Item.Location.Path.Text>
            <axsl:apply-templates select="." mode="schematron-get-full-path-2"/>
            <xsl:if test="string-length( sbdm:MessageEventItem/sbdm:Locations/sbdm:Location/sbdm:Message.Event.Item.Location.Path.Text ) &gt; 0">
              <xsl:text disable-output-escaping="yes">&lt;xsl:text&gt;</xsl:text>/<xsl:value-of select="sbdm:MessageEventItem/sbdm:Locations/sbdm:Location/sbdm:Message.Event.Item.Location.Path.Text" /><xsl:text disable-output-escaping="yes">&lt;/xsl:text&gt;</xsl:text>
            </xsl:if>
          </sbdm:Message.Event.Item.Location.Path.Text>
        </sbdm:Location>
      </sbdm:Locations>

    </sbdm:MessageEventItem>
  </xsl:template>
</xsl:stylesheet>
